-- Set FILESTREAM access level
EXEC sp_configure filestream_access_level, 2
RECONFIGURE


-- Creating a FileTable-enabled database
CREATE DATABASE PhotoFileLibrary
 ON PRIMARY
  (NAME = PhotoFileLibrary_data,
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoFileLibrary_data.mdf'),
 FILEGROUP FileStreamGroup CONTAINS FILESTREAM
  (NAME = PhotoFileLibrary_blobs,
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoFiles')
 LOG ON
  (NAME = PhotoFileLibrary_log,
   FILENAME = 'C:\Demo\PhotoLibrary\PhotoFileLibrary_log.ldf')
 WITH FILESTREAM
  (DIRECTORY_NAME='PhotoFileLibrary',
   NON_TRANSACTED_ACCESS=FULL)


-- Creating a FileTable
USE PhotoFileLibrary
GO

CREATE TABLE PhotoFiles AS FileTable
GO


/* Add a folder and two files */

-- Get root pathnames for the database and FileTable
DECLARE @DbRootPath varchar(max) = FILETABLEROOTPATH()  -- \\machine\instance\db
DECLARE @TableRootPath varchar(max) = @DbRootPath + '\PhotoFiles'

-- Create folder 'Mountains' in the FileTables root
DECLARE @TableRoot hierarchyid = GETPATHLOCATOR(@TableRootPath)
INSERT INTO PhotoFiles(name, path_locator, is_directory) VALUES
 ('Mountains', @TableRoot.GetDescendant(NULL, NULL), 1)

-- Get new folder's hierarchyid
DECLARE @FolderPath varchar(max) = @TableRootPath + '\Mountains'
DECLARE @FolderNode hierarchyid = GETPATHLOCATOR(@FolderPath)

-- Add a text file to the folder
DECLARE @TextFileNode hierarchyid = @FolderNode.GetDescendant(NULL, NULL)
INSERT INTO PhotoFiles(file_stream, name, path_locator) VALUES(
 CONVERT(varbinary(max), 'This folder contains pictures of mountains.'),
 'ReadMe.txt', @TextFileNode)

-- Add an image file to the folder
DECLARE @ImageFileNode hierarchyid = @FolderNode.GetDescendant(@TextFileNode, NULL)
INSERT INTO PhotoFiles(file_stream, name, path_locator) VALUES(
 (SELECT BulkColumn FROM OPENROWSET(BULK 'C:\Demo\Ascent.jpg', SINGLE_BLOB) AS x),
 'Ascent.jpg', @ImageFileNode)

-- Cleanup
DROP TABLE PhotoFiles
