﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Transactions;
using System.Windows.Forms;

namespace DemoRawDataAccess
{
  public partial class Form1 : Form
  {
    public Form1()
    {
      InitializeComponent();
    }

    private void btnDirectSql_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string TSql =
        "INSERT INTO Customer (FirstName, LastName) VALUES ('Lukas', 'Keller')";

      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = TSql;
          cmd.ExecuteNonQuery();
        }
        conn.Close();
      }
    }

    private void btnParameterizedDirectSql_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string TSql =
        "INSERT INTO Customer (FirstName, LastName) VALUES (@FirstName, @LastName)";

      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = TSql;
          cmd.Parameters.AddWithValue("@FirstName", this.txtFirstName.Text);
          cmd.Parameters.AddWithValue("@LastName", this.txtLastName.Text);
          cmd.ExecuteNonQuery();
        }
        conn.Close();
      }
    }

    private void btnStoredProcedureInsert_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string StoredProcName = "InsertCustomer";

      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = StoredProcName;
          cmd.CommandType = CommandType.StoredProcedure;
          cmd.Parameters.AddWithValue("@FirstName", this.txtFirstName.Text);
          cmd.Parameters.AddWithValue("@LastName", this.txtLastName.Text);
          cmd.ExecuteNonQuery();
        }
        conn.Close();
      }
    }

    private void btnStoredProcedureInsertWithReader_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string StoredProcName = "InsertCustomer";

      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = StoredProcName;
          cmd.CommandType = CommandType.StoredProcedure;
          cmd.Parameters.AddWithValue("@FirstName", this.txtFirstName.Text);
          cmd.Parameters.AddWithValue("@LastName", this.txtLastName.Text);
          using (var rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
          {
            rdr.Read();
            var customerId = rdr.GetInt64(0);
            var createdAt = rdr.GetDateTime(1);
            var updatedAt = rdr.GetDateTime(2);
            rdr.Close();
            MessageBox.Show(
              string.Format("Customer ID {0}; Created at {1}; Updated at {2}",
                customerId, createdAt, updatedAt));
          }
        }
      }
    }

    private void btnStoredProcedureWithReader_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string StoredProcName = "SelectCustomers";

      var names = new List<string>();
      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = StoredProcName;
          cmd.CommandType = CommandType.StoredProcedure;
          using (var rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
          {
            while (rdr.Read())
            {
              var name = string.Format("{0}, {1}", rdr["LastName"], rdr["FirstName"]);
              names.Add(name);
            }
            rdr.Close();
          }
        }
      }
      this.lstNames.DataSource = names;
    }

    private void btnOutputParameter_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string StoredProcName = "GetCustomerBalance";

      var customerId = Convert.ToInt64(this.txtCustomerId.Text);
      decimal balance;
      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var cmd = new SqlCommand())
        {
          cmd.Connection = conn;
          cmd.CommandText = StoredProcName;
          cmd.CommandType = CommandType.StoredProcedure;
          cmd.Parameters.AddWithValue("@CustomerId", customerId);
          var outputParam = new SqlParameter()
          {
            ParameterName = "@Balance",
            SqlDbType = SqlDbType.Money,
            Direction = ParameterDirection.Output
          };
          cmd.Parameters.Add(outputParam);
          cmd.ExecuteNonQuery();
          balance = (decimal)outputParam.Value;
        }
        conn.Close();
      }
      MessageBox.Show(string.Format("Customer ID {0} balance is {1:c}",
        customerId, balance));
    }

    private void btnUpdateWithExplicitTransaction_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string TSql =
        "UPDATE Customer SET Balance += @Transfer WHERE CustomerId = @CustomerId";

      var fromCustomerId = Convert.ToInt64(this.txtTransferFromCustomerId.Text);
      var toCustomerId = Convert.ToInt64(this.txtTransferToCustomerId.Text);
      var transferAmount = Convert.ToDecimal(this.txtTransferAmount.Text);

      using (var conn = new SqlConnection())
      {
        conn.ConnectionString = ConnStr;
        conn.Open();
        using (var txn = conn.BeginTransaction())
        {
          try
          {
            using (var cmd1 = new SqlCommand())
            {
              cmd1.Connection = conn;
              cmd1.CommandText = TSql;
              cmd1.Transaction = txn;
              cmd1.Parameters.AddWithValue("@CustomerId", fromCustomerId);
              cmd1.Parameters.AddWithValue("@Transfer", decimal.Negate(transferAmount));
              cmd1.ExecuteNonQuery();
            }
            using (var cmd2 = new SqlCommand())
            {
              cmd2.Connection = conn;
              cmd2.CommandText = TSql;
              cmd2.Transaction = txn;
              cmd2.Parameters.AddWithValue("@CustomerId", toCustomerId);
              cmd2.Parameters.AddWithValue("@Transfer", transferAmount);
              cmd2.ExecuteNonQuery();
            }
            txn.Commit();
          }
          catch
          {
            txn.Rollback();
            throw;
          }
        }
        conn.Close();
      }
    }

    private void btnUpdateWithImplicitTransaction_Click(object sender, EventArgs e)
    {
      const string ConnStr =
        "Data Source=localhost;Initial Catalog=SampleDb;Integrated Security=true;";

      const string TSql =
        "UPDATE Customer SET Balance += @Transfer WHERE CustomerId = @CustomerId";

      var fromCustomerId = Convert.ToInt64(this.txtTransferFromCustomerId.Text);
      var toCustomerId = Convert.ToInt64(this.txtTransferToCustomerId.Text);
      var transferAmount = Convert.ToDecimal(this.txtTransferAmount.Text);

      using (var ts = new TransactionScope())
      {
        using (var conn = new SqlConnection())
        {
          conn.ConnectionString = ConnStr;
          conn.Open();
          using (var cmd1 = new SqlCommand())
          {
            cmd1.Connection = conn;
            cmd1.CommandText = TSql;
            cmd1.Parameters.AddWithValue("@CustomerId", fromCustomerId);
            cmd1.Parameters.AddWithValue("@Transfer", decimal.Negate(transferAmount));
            cmd1.ExecuteNonQuery();
          }
          using (var cmd2 = new SqlCommand())
          {
            cmd2.Connection = conn;
            cmd2.CommandText = TSql;
            cmd2.Parameters.AddWithValue("@CustomerId", toCustomerId);
            cmd2.Parameters.AddWithValue("@Transfer", transferAmount);
            cmd2.ExecuteNonQuery();
          }
          conn.Close();
        }
        ts.Complete();
      }
    }
  }
}
